@@ -360,6 +360,7 @@ static const char decimal_table[200] = {
 };
 } // namespace internal
 
+
 template <typename number_type, typename>
 simdjson_inline void string_builder::append(number_type v) noexcept {
   static_assert(std::is_same<number_type, bool>::value ||
@@ -505,6 +506,7 @@ simdjson_inline void string_builder::append_raw(const char *str,
 #if SIMDJSON_SUPPORTS_CONCEPTS
 // Support for optional types (std::optional, etc.)
 template <concepts::optional_type T>
+ requires(!require_custom_serialization<T>)
 simdjson_inline void string_builder::append(const T &opt) {
   if (opt) {
     append(*opt);
@@ -512,9 +514,17 @@ simdjson_inline void string_builder::append(const T &opt) {
     append_null();
   }
 }
+
+
+template <typename T>
+requires(require_custom_serialization<T>)
+simdjson_inline void string_builder::append(const T &val) {
+  serialize(*this, val);
+}
+
 template <typename T>
 requires(std::is_convertible<T, std::string_view>::value ||
-std::is_same<T, const char*>::value )
+std::is_same<T, const char*>::value)
 simdjson_inline void string_builder::append(const T &value) {
   escape_and_append_with_quotes(value);
 }
